@@ -1,26 +1,30 @@
 (**************************************************************************)
 (*                                                                        *)
-(*    Copyright 2012-2013 OCamlPro                                        *)
+(*    Copyright 2012-2015 OCamlPro                                        *)
 (*    Copyright 2012 INRIA                                                *)
 (*                                                                        *)
-(*  All rights reserved.This file is distributed under the terms of the   *)
-(*  GNU Lesser General Public License version 3.0 with linking            *)
-(*  exception.                                                            *)
-(*                                                                        *)
-(*  OPAM is distributed in the hope that it will be useful, but WITHOUT   *)
-(*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY    *)
-(*  or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General Public        *)
-(*  License for more details.                                             *)
+(*  All rights reserved. This file is distributed under the terms of the  *)
+(*  GNU Lesser General Public License version 2.1, with the special       *)
+(*  exception on linking described in the file LICENSE.                   *)
 (*                                                                        *)
 (**************************************************************************)
 
-(** Typed filename manipulation *)
+(** Higher level file and directory name manipulation AND file operations,
+    wrappers on OpamSystem using the filename type *)
 
 (** Basenames *)
-module Base: OpamMisc.ABSTRACT
+module Base: sig
+  include OpamStd.ABSTRACT
+
+  (** Check whether a basename has a given suffix *)
+  val check_suffix: t -> string -> bool
+
+  (** Add a file extension *)
+  val add_extension: t -> string -> t
+end
 
 (** Directory names *)
-module Dir: OpamMisc.ABSTRACT
+module Dir: OpamStd.ABSTRACT
 
 (** Return the current working directory *)
 val cwd: unit -> Dir.t
@@ -28,9 +32,12 @@ val cwd: unit -> Dir.t
 (** Remove a directory *)
 val rmdir: Dir.t -> unit
 
-(** Clean the contents of a directory. *)
+(** Cleans the contents of a directory, but keeps the directory in place. *)
 val cleandir: Dir.t -> unit
 
+(** Removes an empty directory, as well as any empty leading path components *)
+val rmdir_cleanup: Dir.t -> unit
+
 (** Create a directory *)
 val mkdir: Dir.t -> unit
 
@@ -45,6 +52,9 @@ val dirs: Dir.t -> Dir.t list
 (** Evaluate a function in a given directory *)
 val in_dir: Dir.t -> (unit -> 'a) -> 'a
 
+(** Turns an assoc list into an array suitable to be provided as environment *)
+val env_of_list: (string * string) list -> string array
+
 (** Execute a list of commands in a given directory *)
 val exec: Dir.t -> ?env:(string * string) list -> ?name:string ->
   ?metadata:(string * string) list -> ?keep_going:bool -> string list list -> unit
@@ -52,15 +62,19 @@ val exec: Dir.t -> ?env:(string * string) list -> ?name:string ->
 (** Move a directory *)
 val move_dir: src:Dir.t -> dst:Dir.t -> unit
 
-(** Copy a directory *)
+(** Copy directory [src] to [dst], that is, recursively copy the contents of
+    [src] into [dst], overwriting any existing files. *)
 val copy_dir: src:Dir.t -> dst:Dir.t -> unit
 
 (** Link a directory *)
-val link_dir: src:Dir.t -> dst:Dir.t -> unit
+val link_dir: target:Dir.t -> link:Dir.t -> unit
 
-(** Does the directory existsb ? *)
+(** Does the directory exist? *)
 val exists_dir: Dir.t -> bool
 
+(** Returns the argument as option, if the directory exists *)
+val opt_dir: Dir.t -> Dir.t option
+
 (** Return the parent directory *)
 val dirname_dir: Dir.t -> Dir.t
 
@@ -70,13 +84,16 @@ val basename_dir: Dir.t -> Base.t
 (** Turn a full path into a list of directory names *)
 val to_list_dir: Dir.t -> Dir.t list
 
-(** Creation from a raw string (as {i http://<path>}) *)
+(** Creation from a raw string, without resolving symlinks etc. *)
 val raw_dir: string -> Dir.t
 
 (** Execute a function in a temp directory *)
 val with_tmp_dir: (Dir.t -> 'a) -> 'a
 
-include OpamMisc.ABSTRACT
+(** Provide an automatically cleaned up temp directory to a job *)
+val with_tmp_dir_job: (Dir.t -> 'a OpamProcess.job) -> 'a OpamProcess.job
+
+include OpamStd.ABSTRACT
 
 (** Generic filename *)
 type generic_file =
@@ -90,7 +107,7 @@ val create: Dir.t -> Base.t -> t
     as dirname *)
 val of_basename: Base.t -> t
 
-(** Creation from a raw string (as {i http://<path>}) *)
+(** Creation from a raw string, without resolving symlinks, etc. *)
 val raw: string -> t
 
 (** Prettify a filename:
@@ -121,13 +138,17 @@ val remove: t -> unit
 (** Removes everything in [filename] if existed, then write [contents] instead. *)
 val write: t -> string -> unit
 
-(** see [Sys.file_exists] *)
+(** Returns true if the file exists and is a regular file or a symlink to one *)
 val exists: t -> bool
 
+(** Returns the argument as option if it exists and is either a regular file or
+    a symlink to one *)
+val opt_file: t -> t option
+
 (** Check whether a file has a given suffix *)
 val check_suffix: t -> string -> bool
 
-(** Add a file extension *)
+(** Adds a dot and the given file extension *)
 val add_extension: t -> string -> t
 
 (** Remove the file extension *)
@@ -150,16 +171,15 @@ val copy_in: ?root:Dir.t -> t -> Dir.t -> unit
 (** Move a file *)
 val move: src:t -> dst:t -> unit
 
-(** Symlink a file in a directory *)
-val link_in: t -> Dir.t -> unit
-
 (** Read a symlinked file *)
 val readlink: t -> t
 
-(** Is a symlink ? *)
+(** Is a symlink? *)
 val is_symlink: t -> bool
 
-(** Is an executable ? *)
+val is_symlink_dir: Dir.t -> bool
+
+(** Is an executable? *)
 val is_exec: t -> bool
 
 (** Copy a file *)
@@ -169,16 +189,23 @@ val copy: src:t -> dst:t -> unit
     be set executable *)
 val install: ?exec:bool -> src:t -> dst:t -> unit -> unit
 
-(** Symlink a file. If symlink is not possible on the system, use copy instead. *)
-val link: src:t -> dst:t -> unit
+(** Symlink a file. If symlink is not possible on the system, use copy instead.
+    With [relative], creates a relative link through the closest common ancestor
+    directory if possible. Otherwise, the symlink is absolute. *)
+val link: ?relative:bool -> target:t -> link:t -> unit
 
 (** Extract an archive in a given directory (it rewrites the root to
     match [Dir.t] dir if needed) *)
 val extract: t -> Dir.t -> unit
 
-(** Extract an archive in a given directory (which should already exists) *)
+(** Same as [extract], as an OpamProcess.job *)
+val extract_job: t -> Dir.t -> exn option OpamProcess.job
+
+(** Extract an archive in a given directory *)
 val extract_in: t -> Dir.t -> unit
 
+val extract_in_job: t -> Dir.t -> exn option OpamProcess.job
+
 (** Extract a generic file *)
 val extract_generic_file: generic_file -> Dir.t -> unit
 
@@ -188,37 +215,23 @@ val starts_with: Dir.t -> t -> bool
 (** Check whether a filename ends with a given suffix *)
 val ends_with: string -> t -> bool
 
+(** [dir starts_with pfx dir] Check whether [dir] starts with [pfx] *)
+val dir_starts_with: Dir.t -> Dir.t -> bool
+
+(** Check whether a dirname ends with a given suffix *)
+val dir_ends_with: string -> Dir.t -> bool
+
 (** Remove a prefix from a file name *)
 val remove_prefix: Dir.t -> t -> string
 
+val remove_prefix_dir: Dir.t -> Dir.t -> string
+
 (** Remove a suffix from a filename *)
 val remove_suffix: Base.t -> t -> string
 
-(** download a remote file in a given directory. Return the location
-    of the downloaded file if the download is successful.
-    Compress activates http content compression if supported. May break
-    on gzipped files, only use for text files *)
-val download: overwrite:bool -> ?compress:bool -> t -> Dir.t -> t
-
-(** same as [download], but with a specified destination filename instead of a
-    directory *)
-val download_as: overwrite:bool -> ?compress:bool -> t -> t -> unit
-
-(** iterate downloads until one is sucessful *)
-val download_iter: overwrite:bool -> t list -> Dir.t -> t
-
-(** Apply a patch to a directory *)
-val patch: t -> Dir.t -> unit
-
-(** Compute the MD5 digest of a file *)
-val digest: t -> string
-
-(** Compute the MD5 digest a file. Return the empty list if the file
-    does not exist. *)
-val checksum: t -> string list
-
-(** Compute the MD5 digest for all files in a directory. *)
-val checksum_dir: Dir.t -> string list
+(** Apply a patch in a directory. If [preprocess] is set to false, there is no
+    CRLF translation. Returns [None] on success, the process error otherwise *)
+val patch: ?preprocess:bool -> t -> Dir.t -> exn option OpamProcess.job
 
 (** Create an empty file *)
 val touch: t -> unit
@@ -226,15 +239,35 @@ val touch: t -> unit
 (** Change file permissions *)
 val chmod: t -> int -> unit
 
-(** File locks *)
-val with_flock: ?read:bool -> t -> ('a -> 'b) -> 'a -> 'b
+(** Returns the closest parent of a directory (including itself) for which the
+    predicate holds, if any *)
+val find_in_parents: (Dir.t -> bool) ->  Dir.t -> Dir.t option
 
-(** [copy_if_check t src dst] copies all the files from one directory
-    to another. Do nothing if OPAMDONOTCOPYFILE is set to a non-empty
-    value. *)
-val copy_files: src:Dir.t -> dst:Dir.t -> unit
+(** {2 Locking} *)
 
-module OP: sig
+(** See [OpamSystem.flock]. Prefer the higher level [with_flock] functions when
+    possible *)
+val flock: [< OpamSystem.lock_flag ] -> ?dontblock:bool -> t -> OpamSystem.lock
+
+(** Calls [f] while holding a lock file. Ensures the lock is properly released
+    on [f] exit. Raises [OpamSystem.Locked] if [dontblock] is set and the lock
+    can't be acquired. [f] is passed the file_descr of the lock. *)
+val with_flock: [< OpamSystem.lock_flag ] -> ?dontblock:bool -> t ->
+  (Unix.file_descr -> 'a) -> 'a
+
+(** Calls [f] with the file lock upgraded to at least [flag], then restores the
+    previous lock level. Upgrade to [`Lock_write] should never be used in
+    blocking mode as it would deadlock. Raises [OpamSystem.Locked] (but keeps
+    the lock as is) if [dontblock] is set and the lock can't be upgraded. *)
+val with_flock_upgrade:
+  [< OpamSystem.actual_lock_flag ] -> ?dontblock:bool -> OpamSystem.lock -> (Unix.file_descr -> 'a) -> 'a
+
+(** Runs first function with a write lock on the given file, then releases it to
+    a read lock and runs the second function. *)
+val with_flock_write_then_read:
+  ?dontblock:bool -> t -> (Unix.file_descr -> 'a) -> ('a -> 'b) -> 'b
+
+module Op: sig
 
   (** Create a new directory *)
   val (/): Dir.t -> string -> Dir.t
@@ -247,7 +280,7 @@ end
 (** Simple structure to hanle file attributes *)
 module Attribute: sig
 
-  include OpamMisc.ABSTRACT
+  include OpamStd.ABSTRACT
 
   val to_string_list: t -> string list
 
@@ -257,13 +290,13 @@ module Attribute: sig
   val base: t -> Base.t
 
   (** MD5 digest of the remote file *)
-  val md5: t -> string
+  val md5: t -> OpamHash.t
 
   (** File permission *)
   val perm: t -> int option
 
   (** Constructor*)
-  val create: Base.t -> string -> int -> t
+  val create: Base.t -> OpamHash.t -> int option -> t
 
 end
 
